# -*- coding: utf8 -*-
__author__="garyd"
__date__ ="$Aug 12, 2009 2:59:57 PM$"

import os
import id3
import sys
import logging

# more tests to come.

def test_individual_frame_larger_than_all_frames():
    """corrupt id3 with a frame that declares its size to be larger than the declared frame size."""
    try:
        tags = id3.PathId3("testdata/test_individual_frame_larger_than_all_frames.bin")
        raise AssertionError("That file was invalid.")
    except id3.Mp3Exception:
        pass

def test_invalid_text_frame_encoding():
    """this mp3 has 0x03 listed as a text encoding. valid values are 0x00 and 0x01."""
    try:
        tags = id3.PathId3("testdata/test_invalid_text_frame_encoding.bin")
    except:
        raise AssertionError("Should have read that file.")

def test_apic_with_nonexisting_description():
    """the spec is underspecified in this regard. apparently the APIC description is optional. It could be there
    or it could not be there, but there is no good way to tell. Solution is to employ heuristics."""
    try:
        tags = id3.PathId3("testdata/test_apic_with_nonexisting_description.bin")
    except:
        raise AssertionError("Should have read that file.")

def test_iso8859_high_order_bytes():
    """there is a strange problem when printing out tags.artist (has unicode 'u' in it)."""
    try:
        tags = id3.PathId3("testdata/test_iso8859_high_order_bytes.bin")
        assert tags.artist == u"Hüsker Dü"
    except:
        print ex_to_str()
        raise AssertionError("Should have read that file.")
    

###
### If you're examining this code for kicks and giggles, everything below this point exists
### for development purposes only. They don't deal too much with testing except as it relates
### to developing the tests themselves.
###

def extract_tags_in_preparation_for_tseting():
    """Not a test.  I use this on my development system to extract tags from Mp3s I want to test with."""
    tups = zip(
        [
            "test_individual_frame_larger_than_all_frames",
            "test_invalid_text_frame_encoding",
            "test_apic_with_nonexisting_description",
            "test_iso8859_high_order_bytes"
        ],
        [
            "/mnt/hgfs/from_server/mp3/singles/Aerosmith/Aerosmith, Janie's got a gun.mp3",
            "/mnt/hgfs/from_server/mp3/singles/Clementines/Peeler_ for Left or Right-Handed Use/04 Super Showbiz Star.mp3",
            "/mnt/hgfs/from_server/old_itunes/The Decemberists/Picaresque/01 The Infanta.mp3",
            "/mnt/hgfs/from_server/mp3/singles/Husker Du/Husker Du - 07. Friend, You've Got To Fall.mp3"
        ]
    )
    for extracted_name, path in tups:
        id3.extract_binary_tag(file(path,"rb"), "testdata/" + extracted_name + ".bin")

def scan_them():
    """Not a test. Scans my mp3 samples for smoke testing. I run this prior to any release."""
    count = 0
    for path in mp3s(MP3_DIR):
        print "\n" + str(count) + " TRYING " + path
        try:
            tags = id3.PathId3(path)
            for tag in ["album", "artist", "track", "title", "year"]:
                try:
                    print "%s: %s" % (tag, getattr(tags, tag))
                except UnicodeEncodeError:
                    print "Unicode problems"
            if tags.picture:
                print "HAS PICTURE"
            else:
                print "NO PICTURE"
                
        except id3.Mp3Exception:
            # these kinds of exceptions are raised in well documented places
            # where recovery is possible. This usually indicates corrupted data
            # in the mp3.
            print "Unrecoverable error on %s" % path
        except:
            # Execution should never get there. This represents a failure of the library
            # to handle somethign properly. Investigate and fix.
            print "error on %s" % path
            print ex_to_str()
            sys.exit(-1)

MP3_DIR = "/mnt/hgfs/from_server"

def mp3s(path):
    """generator for files."""
    for root, dirs, files in os.walk(path):
        for file in files:
            if file.lower().endswith(".mp3"):
                yield os.path.join(root, file)
                
import traceback
import StringIO
def ex_to_str():
    err = StringIO.StringIO()
    traceback.print_exc(file=err)
    return err.getvalue()

if __name__ == "__main__":
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    #id3.logger.addHandler(ch)
    scan_them()
    #extract_tags_in_preparation_for_testing()
    #test_unicode_text_frames()

    